<?php

namespace NoccyLabs\TinyPhar;

class Package
{
    /** @var string Package name */
    protected $name;
    /** @var string The version number as SEMVER */
    protected $version;
    /** @var string The URL where the package file can be found */
    protected $url;
    /** @var string Sha1 checksum for the file at $url */
    protected $sha1;
    /** @var string Summary information on this release */
    protected $info;
    /** @var string URL where more information can be found on this release */
    protected $info_url;

    public function __construct($name, $version, $url=null, $info=null)
    {
        $this->name = $name;
        $this->version = $version;
        $this->setUrl($url);
        $this->setInfo($info);
    }
    
    public function setUrl($url, $sha1=null)
    {
        $this->url = $url;
        $this->sha1 = $sha1;
    }
    
    public function setInfo($info, $info_url=null)
    {
        $this->info = $info;
        $this->info_url = $info_url;
    }
    
    public function getName()
    {
        return $this->name;
    }
    
    public function getVersion()
    {
        return $this->version;
    }
    
    public function getUrl()
    {
        return $this->url;
    }
    
    public function getInfo()
    {
        return $this->info;
    }
    
    public function compareVersion($compare_to)
    {
        return version_compare($this->version, $compare_to);
    }
    
    public function isNewerThan($compare_to)
    {
        return ($this->compareVersion($compare_to) > 0);
    }
}

