<?php

use NoccyLabs\TinyPhar\Updater;

if (defined("IS_PHAR")) {
    // We have already been loaded!
    return;
}

if (($phar = Phar::running())) {
    // -- running inside phar ---------------------------------------
    define("IS_PHAR", true);
    define("PHAR_PATH", $phar);
    define("PHAR_DIR", dirname($phar));
    define("PHAR_FILE", realpath($GLOBALS['argv'][0]));
    // Read the metadata
    $obj = new Phar($phar);
    $meta = $obj->getMetadata();
    foreach ($meta as $k=>$v) {
        define("META_".strtoupper($k),$v);
    }
} else {
    // -- running outside of phar -----------------------------------
    define("IS_PHAR", false);
}
// APP_DIR: The directory where the application lives
define("APP_DIR", IS_PHAR?realpath($GLOBALS['argv'][0]):dirname(realpath($GLOBALS['argv'][0])));


/**
 * Check for updates based on  name, version and chanel url
 *
 * @param string $name Package name
 * @param string $version Version number
 * @param string $list The channel URL to check for updates
 */
function tinyphar_update($name, $version, $list) {

    if (!IS_PHAR) return;

    $updater = new Updater();
    $updater->setPackageInfo($name, $version);

    $updater->readPackageList($list);

    if ($updater->isLatestVersion()) {
        printf("\e[32m ▸ The installed version \e[1m%s\e[21m is the latest version!\e[0m\n", $version);
    } else {
        $latest = $updater->getLatestVersion();
        printf("\e[32m ▸ Updating version \e[1m%s\e[21m to version \e[1m%s\e[21m...\e[0m\n", $version, $latest->getVersion());
        printf("\e[36m ▸ Downloading \e[1m%s\e[21m ...\e[0m\n", $latest->getUrl());

        $lastphar = dirname(PHAR_FILE)."/.".basename(PHAR_FILE).".last";
        $nextphar = dirname(PHAR_FILE)."/.".basename(PHAR_FILE).".new";
        $updater->downloadVersion($latest, $nextphar);

        printf("\e[36m ▸ Writing \e[1m%s\e[21m ...\e[0m\n", PHAR_FILE);
        $mode = fileperms(PHAR_FILE);
        if (file_exists($lastphar)) {
            unlink($lastphar);
        }
        rename(PHAR_FILE, $lastphar);
        rename($nextphar, PHAR_FILE);
        chmod(PHAR_FILE, $mode);

        printf("\e[32m ▸ Successfully updated to version \e[1m%s\e[21m!\e[0m\n", $latest->getVersion());
    }

}

/** 
 * Automatically update if --self-update is passed on the command line.
 * A suitable define must be present for the package name, version, and
 * the update channel to use.
 *
 * If the 'package_name', 'package_version' and 'update_channel'' are
 * set on the .phar, these will be picked up automatically.
 */
function tinyphar_autoupdate() {
    // Make sure we have --self-update on the command line
    $opts = getopt("",["self-update"]);
    if (!array_key_exists("self-update",$opts)) {
        return;
    }
    // Also make sure that we are running from a phar
    if (!IS_PHAR) {
        echo "You can only --self-update phars\n";
        exit(1);
    }
    // Helper to grab the first matching define
    $_ = function(...$defines) {
        foreach ($defines as $define) {
            if(defined($define)) return constant($define);
        }
        return null;
    };

    $pkg_name = $_("META_PACKAGE_NAME","PACKAGE_NAME");
    $pkg_version = $_("META_PACKAGE_VERSION","PACKAGE_VERSION","APP_VERSION");
    $pkg_channel = $_("META_UPDATE_CHANNEL","UPDATE_CHANNEL");
    
    if ((!$pkg_name) || (!$pkg_version) || (!$pkg_channel)) {
        echo "This .phar could not be autoupdated, required metadata not set.\n";
        exit(1);
    }

    tinyphar_update($pkg_name, $pkg_version, $pkg_channel);

    exit(0);
}
// define TINYPHAR_NO_AUTOUPDATE as false to prevent autoupdate
if ((!defined("TINYPHAR_NO_AUTOUPDATE")) || (TINYPHAR_NO_AUTOUPDATE==false)) {
    tinyphar_autoupdate();
}
